@@ -26,8 +26,8 @@ pub fn run(arg: PathBuf) {
     }
 
     let mut state = State::new();
-    let mut current_dir = arg.canonicalize().unwrap();
-    state.update_list(&current_dir);
+    state.current_dir = arg.canonicalize().unwrap();
+    state.update_list();
     state.trash_dir = trash_dir;
 
     let mut nums = Num::new();
@@ -41,7 +41,7 @@ pub fn run(arg: PathBuf) {
 
     print!("{}", cursor::Hide);
 
-    clear_and_show(&current_dir);
+    clear_and_show(&state.current_dir);
     state.list_up(nums.skip);
 
     print!("{}>{}", cursor::Goto(1, STARTING_POINT), cursor::Left(1));
@@ -63,7 +63,7 @@ pub fn run(arg: PathBuf) {
                         continue;
                     } else if y == row - 4 && len > (row - STARTING_POINT) as usize - 1 {
                         nums.inc_skip();
-                        clear_and_show(&current_dir);
+                        clear_and_show(&state.current_dir);
                         state.list_up(nums.skip);
                         print!("{}>{}", cursor::Goto(1, y), cursor::Left(1));
                         nums.go_down();
@@ -79,7 +79,7 @@ pub fn run(arg: PathBuf) {
                         continue;
                     } else if y == STARTING_POINT + 3 && nums.skip != 0 {
                         nums.dec_skip();
-                        clear_and_show(&current_dir);
+                        clear_and_show(&state.current_dir);
                         state.list_up(nums.skip);
                         print!(
                             "{}>{}",
@@ -111,7 +111,7 @@ pub fn run(arg: PathBuf) {
                                     Key::Char('g') => {
                                         print!("{}", cursor::Hide);
                                         nums.reset();
-                                        clear_and_show(&current_dir);
+                                        clear_and_show(&state.current_dir);
                                         state.list_up(0);
                                         print!(
                                             " {}>{}",
@@ -145,7 +145,7 @@ pub fn run(arg: PathBuf) {
                     }
                     if len > (row - STARTING_POINT) as usize {
                         nums.skip = (len as u16) + STARTING_POINT - row;
-                        clear_and_show(&current_dir);
+                        clear_and_show(&state.current_dir);
                         state.list_up(nums.skip);
                         print!("{}>{}", cursor::Goto(1, row - 1), cursor::Left(1));
                     } else {
@@ -169,7 +169,7 @@ pub fn run(arg: PathBuf) {
                                     continue;
                                 }
                                 print!("{}", screen::ToAlternateScreen);
-                                clear_and_show(&current_dir);
+                                clear_and_show(&state.current_dir);
                                 state.list_up(nums.skip);
                                 print!(
                                     "{}{}>{}",
@@ -192,11 +192,15 @@ pub fn run(arg: PathBuf) {
                                         };
                                         memo_v.push(cursor_memo);
 
-                                        current_dir = item.file_path.clone();
-                                        std::env::set_current_dir(&current_dir)
-                                            .unwrap_or_else(|e| print_warning(e, y));
-                                        state.update_list(&current_dir);
-                                        clear_and_show(&current_dir);
+                                        state.current_dir = item.file_path.clone();
+                                        if let Err(e) =
+                                            std::env::set_current_dir(&state.current_dir)
+                                        {
+                                            print_warning(e, y);
+                                            continue;
+                                        }
+                                        state.update_list();
+                                        clear_and_show(&state.current_dir);
                                         state.list_up(0);
                                         print!(
                                             "{}>{}",
@@ -212,13 +216,13 @@ pub fn run(arg: PathBuf) {
                 }
 
                 //Go to parent directory if exists
-                Key::Char('h') | Key::Left => match current_dir.parent() {
+                Key::Char('h') | Key::Left => match state.current_dir.parent() {
                     Some(parent_p) => {
-                        current_dir = parent_p.to_path_buf();
-                        std::env::set_current_dir(&current_dir)
+                        state.current_dir = parent_p.to_path_buf();
+                        std::env::set_current_dir(&state.current_dir)
                             .unwrap_or_else(|e| print_warning(e, y));
-                        state.update_list(&current_dir);
-                        clear_and_show(&current_dir);
+                        state.update_list();
+                        clear_and_show(&state.current_dir);
                         state.list_up(0);
 
                         match memo_v.pop() {
@@ -244,7 +248,7 @@ pub fn run(arg: PathBuf) {
                     let mut item = state.list.get_mut(nums.index).unwrap();
                     item.selected = true;
 
-                    clear_and_show(&current_dir);
+                    clear_and_show(&state.current_dir);
                     state.list_up(nums.skip);
                     print!("{}>{}", cursor::Goto(1, y), cursor::Left(1));
                     screen.flush().unwrap();
@@ -274,7 +278,7 @@ pub fn run(arg: PathBuf) {
                                             item.selected = false;
                                         }
 
-                                        clear_and_show(&current_dir);
+                                        clear_and_show(&state.current_dir);
                                         state.list_up(nums.skip);
                                         print!("{}>{}", cursor::Goto(1, y), cursor::Left(1));
                                         screen.flush().unwrap();
@@ -290,7 +294,7 @@ pub fn run(arg: PathBuf) {
                                             item.selected = false;
                                         }
 
-                                        clear_and_show(&current_dir);
+                                        clear_and_show(&state.current_dir);
                                         state.list_up(nums.skip);
                                         print!("{}>{}", cursor::Goto(1, y + 1), cursor::Left(1));
                                         screen.flush().unwrap();
@@ -314,7 +318,7 @@ pub fn run(arg: PathBuf) {
                                             item.selected = true;
                                         }
 
-                                        clear_and_show(&current_dir);
+                                        clear_and_show(&state.current_dir);
                                         state.list_up(nums.skip);
                                         print!(
                                             "{}>{}",
@@ -334,7 +338,7 @@ pub fn run(arg: PathBuf) {
                                             item.selected = true;
                                         }
 
-                                        clear_and_show(&current_dir);
+                                        clear_and_show(&state.current_dir);
                                         state.list_up(nums.skip);
                                         print!("{}>{}", cursor::Goto(1, y - 1), cursor::Left(1));
                                         screen.flush().unwrap();
@@ -360,7 +364,7 @@ pub fn run(arg: PathBuf) {
                                                         print!("{}", cursor::Hide);
                                                         nums.reset();
                                                         state.select_from_top(start_pos);
-                                                        clear_and_show(&current_dir);
+                                                        clear_and_show(&state.current_dir);
                                                         state.list_up(0);
                                                         print!(
                                                             " {}>{}",
@@ -396,13 +400,13 @@ pub fn run(arg: PathBuf) {
                                         nums.skip = (len as u16) + STARTING_POINT - row;
                                         nums.go_bottom(len - 1);
                                         state.select_to_bottom(start_pos);
-                                        clear_and_show(&current_dir);
+                                        clear_and_show(&state.current_dir);
                                         state.list_up(nums.skip);
                                         print!("{}>{}", cursor::Goto(1, row - 1), cursor::Left(1));
                                     } else {
                                         nums.go_bottom(len - 1);
                                         state.select_to_bottom(start_pos);
-                                        clear_and_show(&current_dir);
+                                        clear_and_show(&state.current_dir);
                                         state.list_up(nums.skip);
                                         print!(
                                             " {}>{}",
@@ -413,6 +417,9 @@ pub fn run(arg: PathBuf) {
                                 }
 
                                 Key::Char('d') => {
+                                    print_info("Processing...", y);
+                                    screen.flush().unwrap();
+
                                     state.registered.clear();
                                     let iter = state.list.clone().into_iter();
                                     let mut i = 0;
@@ -423,20 +430,22 @@ pub fn run(arg: PathBuf) {
                                                     if let Err(e) = state.remove_and_yank_dir(item)
                                                     {
                                                         print_warning(e, y);
+                                                        break;
                                                     }
                                                 }
                                                 FileType::File | FileType::Symlink => {
                                                     if let Err(e) = state.remove_and_yank_file(item)
                                                     {
                                                         print_warning(e, y);
+                                                        break;
                                                     }
                                                 }
                                             }
                                             i += 1;
                                         }
                                     }
-                                    clear_and_show(&current_dir);
-                                    state.update_list(&current_dir);
+                                    clear_and_show(&state.current_dir);
+                                    state.update_list();
                                     state.list_up(nums.skip);
 
                                     let mut delete_message: String = i.to_string();
@@ -470,7 +479,7 @@ pub fn run(arg: PathBuf) {
                                 Key::Char('y') => {
                                     state.yank_item(nums.index, true);
                                     state.reset_selection();
-                                    clear_and_show(&current_dir);
+                                    clear_and_show(&state.current_dir);
                                     state.list_up(nums.skip);
 
                                     let mut yank_message: String =
@@ -484,7 +493,7 @@ pub fn run(arg: PathBuf) {
 
                                 Key::Esc => {
                                     state.reset_selection();
-                                    clear_and_show(&current_dir);
+                                    clear_and_show(&state.current_dir);
                                     state.list_up(nums.skip);
                                     print!("{}>{}", cursor::Goto(1, y), cursor::Left(1));
                                     break;
@@ -508,8 +517,8 @@ pub fn run(arg: PathBuf) {
                             state.sort_by = SortKey::Name;
                         }
                     }
-                    state.update_list(&current_dir);
-                    clear_and_show(&current_dir);
+                    state.update_list();
+                    clear_and_show(&state.current_dir);
                     state.list_up(0);
                     print!("{}>{}", cursor::Goto(1, STARTING_POINT), cursor::Left(1));
                     nums.reset();
@@ -529,24 +538,32 @@ pub fn run(arg: PathBuf) {
                             if let Some(Ok(key)) = input {
                                 match key {
                                     Key::Char('d') => {
+                                        print_info("Processing...", y);
+                                        screen.flush().unwrap();
+
                                         state.registered.clear();
                                         let item = state.get_item(nums.index).unwrap().clone();
                                         match item.file_type {
                                             FileType::Directory => {
                                                 if let Err(e) = state.remove_and_yank_dir(item) {
+                                                    print!("{}", cursor::Hide);
                                                     print_warning(e, y);
+                                                    break 'delete;
                                                 }
                                             }
                                             FileType::File | FileType::Symlink => {
                                                 if let Err(e) = state.remove_and_yank_file(item) {
+                                                    clear_and_show(&state.current_dir);
+                                                    print!("{}", cursor::Hide);
                                                     print_warning(e, y);
+                                                    break 'delete;
                                                 }
                                             }
                                         }
 
-                                        clear_and_show(&current_dir);
+                                        clear_and_show(&state.current_dir);
                                         print!("{}", cursor::Hide);
-                                        state.update_list(&current_dir);
+                                        state.update_list();
                                         state.list_up(nums.skip);
                                         let cursor_pos = if state.list.is_empty() {
                                             STARTING_POINT
@@ -624,29 +641,21 @@ pub fn run(arg: PathBuf) {
                     if state.registered.is_empty() {
                         continue;
                     }
-                    for item in state.registered.clone().into_iter() {
-                        match item.file_type {
-                            FileType::Directory => {
-                                if let Err(e) = state.put_dir(&item, &current_dir) {
-                                    print_warning(e, y);
-                                }
-                            }
-                            FileType::File | FileType::Symlink => {
-                                if let Err(e) = state.put_file(&item, &current_dir) {
-                                    print_warning(e, y);
-                                }
-                            }
-                        }
+                    print_info("Processing...", y);
+                    screen.flush().unwrap();
+
+                    if let Err(e) = state.put_items() {
+                        print_warning(e, y);
+                        continue;
                     }
-                    clear_and_show(&current_dir);
-                    state.update_list(&current_dir);
+
+                    clear_and_show(&state.current_dir);
+                    state.update_list();
                     state.list_up(nums.skip);
 
                     let mut put_message: String = state.registered.len().to_string();
                     put_message.push_str(" items inserted");
                     print_info(put_message, y);
-
-                    print!("{}>{}", cursor::Goto(1, y), cursor::Left(1));
                 }
 
                 Key::Char('c') => {
@@ -675,17 +684,18 @@ pub fn run(arg: PathBuf) {
                                 //rename item
                                 Key::Char('\n') => {
                                     let rename = rename.iter().collect::<String>();
-                                    let mut to = current_dir.clone();
+                                    let mut to = state.current_dir.clone();
                                     to.push(rename);
                                     if let Err(e) =
                                         std::fs::rename(Path::new(&item.file_path), Path::new(&to))
                                     {
+                                        print!("{}", cursor::Hide);
                                         print_warning(e, y);
                                         break;
                                     }
 
-                                    clear_and_show(&current_dir);
-                                    state.update_list(&current_dir);
+                                    clear_and_show(&state.current_dir);
+                                    state.update_list();
                                     state.list_up(nums.skip);
 
                                     print!(
@@ -817,7 +827,7 @@ pub fn run(arg: PathBuf) {
                                 }
 
                                 Key::Esc => {
-                                    clear_and_show(&current_dir);
+                                    clear_and_show(&state.current_dir);
                                     state.list = original_list;
                                     state.list_up(0);
 
@@ -864,7 +874,7 @@ pub fn run(arg: PathBuf) {
                                         .collect();
 
                                     nums.reset_skip();
-                                    clear_and_show(&current_dir);
+                                    clear_and_show(&state.current_dir);
                                     state.list_up(nums.skip);
 
                                     print!(
@@ -896,7 +906,7 @@ pub fn run(arg: PathBuf) {
                                         .collect();
 
                                     nums.reset_skip();
-                                    clear_and_show(&current_dir);
+                                    clear_and_show(&state.current_dir);
                                     state.list_up(nums.skip);
 
                                     print!(
@@ -946,8 +956,8 @@ pub fn run(arg: PathBuf) {
                                     if command == vec!['q'] {
                                         break 'main;
                                     } else if command == vec!['e'] {
-                                        state.update_list(&current_dir);
-                                        clear_and_show(&current_dir);
+                                        state.update_list();
+                                        clear_and_show(&state.current_dir);
                                         state.list_up(0);
                                         print!(
                                             "{}{}>{}",
@@ -967,7 +977,7 @@ pub fn run(arg: PathBuf) {
                                         }
                                         println!("\nInput any key to go back.");
                                         let _ = stdin.next();
-                                        clear_and_show(&current_dir);
+                                        clear_and_show(&state.current_dir);
                                         state.list_up(nums.skip);
                                         print!("{}>{}", cursor::Goto(1, y), cursor::Left(1));
                                         break 'command;
@@ -998,15 +1008,22 @@ pub fn run(arg: PathBuf) {
                                             if let Some(Ok(key)) = input {
                                                 match key {
                                                     Key::Char('y') | Key::Char('Y') => {
+                                                        print_info("Processing...", y);
+                                                        screen.flush().unwrap();
+
                                                         if let Err(e) = std::fs::remove_dir_all(
                                                             &state.trash_dir,
                                                         ) {
+                                                            print!("{}", cursor::Hide);
                                                             print_warning(e, y);
+                                                            continue 'main;
                                                         }
                                                         if let Err(e) =
                                                             std::fs::create_dir(&state.trash_dir)
                                                         {
+                                                            print!("{}", cursor::Hide);
                                                             print_warning(e, y);
+                                                            continue 'main;
                                                         }
                                                         break 'empty;
                                                     }
@@ -1023,15 +1040,26 @@ pub fn run(arg: PathBuf) {
                                             clear::CurrentLine,
                                             DOWN_ARROW
                                         );
-                                        print!("{}>{}", cursor::Goto(1, y), cursor::Left(1));
+                                        if state.current_dir == state.trash_dir {
+                                            clear_and_show(&state.current_dir);
+                                            state.update_list();
+                                            state.list_up(nums.skip);
+                                            print!(
+                                                "{}>{}",
+                                                cursor::Goto(1, STARTING_POINT),
+                                                cursor::Left(1)
+                                            );
+                                        } else {
+                                            print!("{}>{}", cursor::Goto(1, y), cursor::Left(1));
+                                        }
                                         break 'command;
                                     }
 
                                     // if c == "cd" {
                                     //     current_dir =
                                     //         PathBuf::from(args[0]).canonicalize().unwrap();
-                                    //     state.update_list(&current_dir);
-                                    //     clear_and_show(&current_dir);
+                                    //     state.update_list();
+                                    //     clear_and_show(&state.current_dir);
                                     //     state.list_up(0);
                                     //     print!(
                                     //         "{}{}>{}",
@@ -1044,15 +1072,16 @@ pub fn run(arg: PathBuf) {
                                     // }
 
                                     print!("{}", screen::ToAlternateScreen);
-                                    std::env::set_current_dir(&current_dir)
-                                        .unwrap_or_else(|e| print_warning(e, y));
-                                    if let Err(_) =
-                                        std::process::Command::new(c).args(args).status()
-                                    {
+                                    if std::env::set_current_dir(&state.current_dir).is_err() {
+                                        print!("{}", cursor::Hide,);
+                                        print_warning("cannot execute command", y);
+                                        break 'command;
+                                    }
+                                    if std::process::Command::new(c).args(args).status().is_err() {
                                         print!("{}", screen::ToAlternateScreen);
 
-                                        clear_and_show(&current_dir);
-                                        state.update_list(&current_dir);
+                                        clear_and_show(&state.current_dir);
+                                        state.update_list();
                                         state.list_up(nums.skip);
 
                                         print!("{}", cursor::Hide,);
@@ -1061,8 +1090,8 @@ pub fn run(arg: PathBuf) {
                                     }
                                     print!("{}", screen::ToAlternateScreen);
 
-                                    clear_and_show(&current_dir);
-                                    state.update_list(&current_dir);
+                                    clear_and_show(&state.current_dir);
+                                    state.update_list();
                                     state.list_up(nums.skip);
 
                                     print!(
