@@ -2,6 +2,7 @@ use super::config::*;
 use super::functions::*;
 use chrono::prelude::*;
 use std::collections::HashMap;
+use std::collections::HashSet;
 use std::fs;
 use std::io::{Error, ErrorKind};
 use std::path::{Path, PathBuf};
@@ -49,6 +50,7 @@ macro_rules! print_item {
 pub struct State {
     pub list: Vec<ItemInfo>,
     pub registered: Vec<ItemInfo>,
+    pub current_dir: PathBuf,
     pub trash_dir: PathBuf,
     pub colors: (Colorname, Colorname, Colorname),
     pub default: String,
@@ -78,6 +80,7 @@ impl Default for State {
         State {
             list: Vec::new(),
             registered: Vec::new(),
+            current_dir: PathBuf::new(),
             trash_dir: PathBuf::new(),
             colors: (
                 config.color.dir_fg,
@@ -220,28 +223,50 @@ impl State {
         }
     }
 
-    pub fn put_file(&mut self, item: &ItemInfo, current_dir: &Path) -> std::io::Result<()> {
-        if item.file_path.parent() == Some(&self.trash_dir) {
-            let mut item = item.clone();
-            let rename = item.file_name.chars().skip(11).collect();
-            item.file_name = rename;
-            let rename = rename_file(&item, self);
-            std::fs::copy(&item.file_path, current_dir.join(&rename))?;
-        } else {
-            let rename = rename_file(item, self);
-            std::fs::copy(&item.file_path, current_dir.join(&rename))?;
+    pub fn put_items(&mut self) -> std::io::Result<()> {
+        //make HashSet<String> of file_name
+        let mut name_set = HashSet::new();
+        for item in self.list.iter() {
+            name_set.insert(item.file_name.clone());
+        }
+
+        for item in self.registered.clone().into_iter() {
+            match item.file_type {
+                FileType::Directory => {
+                    self.put_dir(&item, &mut name_set)?;
+                }
+                FileType::File | FileType::Symlink => {
+                    self.put_file(&item, &mut name_set)?;
+                }
+            }
         }
         Ok(())
     }
 
-    pub fn put_dir(&mut self, buf: &ItemInfo, current_dir: &Path) -> std::io::Result<()> {
+    fn put_file(&mut self, item: &ItemInfo, name_set: &mut HashSet<String>) -> std::io::Result<()> {
+        if item.file_path.parent() == Some(&self.trash_dir) {
+            let mut item = item.clone();
+            let rename = item.file_name.chars().skip(11).collect();
+            item.file_name = rename;
+            let rename = rename_file(&item, name_set);
+            std::fs::copy(&item.file_path, &self.current_dir.join(&rename))?;
+            name_set.insert(rename);
+        } else {
+            let rename = rename_file(item, name_set);
+            std::fs::copy(&item.file_path, &self.current_dir.join(&rename))?;
+            name_set.insert(rename);
+        }
+        Ok(())
+    }
+
+    fn put_dir(&mut self, buf: &ItemInfo, name_set: &mut HashSet<String>) -> std::io::Result<()> {
         let mut base: usize = 0;
         let mut target: PathBuf = PathBuf::new();
         let original_path = &(buf).file_path;
 
         let mut i = 0;
         for entry in walkdir::WalkDir::new(&original_path).sort_by_key(|x| x.path().to_path_buf()) {
-            let entry = entry.unwrap();
+            let entry = entry?;
             if i == 0 {
                 base = entry.path().iter().count();
 
@@ -251,11 +276,13 @@ impl State {
                     let rename = buf.file_name.chars().skip(11).collect();
                     buf.file_name = rename;
 
-                    let rename = rename_dir(&buf, self);
-                    target = current_dir.join(rename);
+                    let rename = rename_dir(&buf, name_set);
+                    target = self.current_dir.join(&rename);
+                    name_set.insert(rename);
                 } else {
-                    let rename = rename_dir(buf, self);
-                    target = current_dir.join(rename);
+                    let rename = rename_dir(buf, name_set);
+                    target = self.current_dir.join(&rename);
+                    name_set.insert(rename);
                 }
                 std::fs::create_dir(&target)?;
                 i += 1;
@@ -397,8 +424,8 @@ impl State {
         }
     }
 
-    pub fn update_list(&mut self, path: &Path) {
-        self.list = push_items(path, &self.sort_by).unwrap();
+    pub fn update_list(&mut self) {
+        self.list = push_items(&self.current_dir, &self.sort_by).unwrap();
     }
 
     pub fn reset_selection(&mut self) {
